/**
 * Copyright (c) 2004 Logos GmbH Rostock (www.logos-ib.de).
 * All rights reserved.
 *
 * THIS SOFTWARE IS  PROVIDED  "AS IS"  AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF  MERCHANTABILITY  AND  FITNESS  FOR  A PARTICULAR  PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL  LOGOS GMBH  OR ITS CONTRIBUTORS BE
 * LIABLE FOR ANY  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES  (INCL., BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR  PROFITS;
 * OR BUSINESS INTERRUPTION)  HOWEVER  CAUSED  AND  ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,  OR TORT (INCL.
 * NEGLIGENCE OR OTHERWISE)  ARISING  IN  ANY  WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package de.bsvrz.dua.bastband.bastband.dvs.kex.datenaustausch.output;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.Iterator;
import java.util.List;
import java.util.TreeMap;

/**
 * Logger zur Erzeugung eines Protokolls, das mehrere Calendar-bezogene LogItem
 * enthalten kann.
 * <p>
 * Es wird zwischen Warnungen und Fehlern unterschieden.
 * </p>
 * <p>
 * Warnungen sollten erzeugt werden, wenn die Ursache f&uuml;r das Fehlverhalten
 * bestimmbar ist und dem User eine Meldung &uuml;bergeben werden kann die
 * aussagekr&auml;tig ist.
 * <p>
 * Bei allen anderen Ereignissen sind Fehlereintr&auml;ge zu erzeugen.
 * </p>
 *
 * @author Ralf Trier
 * @version 1.0
 */
public class ChannelLogger {

	private static SimpleDateFormat __DateFormater = new SimpleDateFormat("dd.MM.yyyy HH':00 Uhr'");

	TreeMap<Date, LogItem> _TreeMap = new TreeMap<>();
	String _Title;

	int _ErrorCount = 0;

	public ChannelLogger(final String title) {
		super();
		_Title = title;
	}

	/**
	 * f&uuml;gt eine Warnung hinzu.
	 *
	 * @param date
	 *            Zeitpunkt
	 * @param message
	 *            Object
	 */
	public synchronized void logWarning(final Date date, final String message) {
		LogItem logItem = _TreeMap.get(date);
		if (logItem == null) {
			logItem = new LogItem(date);
			_TreeMap.put(date, logItem);
		}
		logItem.addMessage(message);
	}

	/**
	 * f&uuml;gt ein Fehler hinzu.
	 *
	 * @param date
	 *            Calendar
	 * @param message
	 *            Object
	 * @param throwable
	 *            Throwable
	 */
	public synchronized void logError(final Date date, final String message, final Throwable throwable) {
		LogItem logItem = _TreeMap.get(date);
		if (logItem == null) {
			logItem = new LogItem(date);
			_TreeMap.put(date, logItem);
		}
		logItem.addMessage(message, throwable);
		_ErrorCount++;
	}

	/**
	 * gibt true zur&uuml;ck, wenn Fehler vorhanden sind.
	 *
	 * @return boolean
	 */
	public boolean containsErrors() {
		return (_ErrorCount > 0);
	}

	/**
	 * gibt true zur&uuml;ck, wenn Warnungen oder Fehler vorhanden sind.
	 *
	 * @return boolean
	 */
	public boolean containsWarnings() {
		return (_TreeMap.size() > 0);
	}

	/**
	 * Liefert Fehlermeldungen als Liste von Strings.
	 *
	 * @return String[] Fehlermeldungen, ist null wenn keine Fehler
	 */
	public String[] getText() {
		String[] resArray = null;
		if (_TreeMap.size() > 0) {
			final List<String> list = new ArrayList<>();
			final Iterator<LogItem> iterator = _TreeMap.values().iterator();
			final StringBuffer sb = new StringBuffer();
			while (iterator.hasNext()) {
				final LogItem logItem = iterator.next();
				final String[] sArray = logItem.getText();
				if ((sArray != null) && (sArray.length > 0)) {
					sb.append(ChannelLogger.__DateFormater.format(logItem._Date));
					for (final String element : sArray) {
						while (sb.length() < 25) {
							sb.append(' ');
						}
						sb.append(element);
						list.add(sb.toString());
						sb.setLength(0);
					}
				}
			}
			resArray = list.toArray(new String[list.size()]);
		}
		return resArray;
	}

}

class LogItem implements Comparable<Object> {

	Date _Date;
	ArrayList<Object> _ObjectList = new ArrayList<>(2);

	LogItem(final Date date) {
		_Date = date;
	}

	public void addMessage(final String message) {
		_ObjectList.add(message);
	}

	public void addMessage(final String message, final Throwable throwable) {
		_ObjectList.add(new ExceptionItem(message, throwable));
	}

	@Override
	public boolean equals(final Object o) {
		if ((o != null) && (o instanceof LogItem)) {
			return (_Date.equals(((LogItem) o)._Date));
		}
		return false;
	}

	@Override
	public int compareTo(final Object o) {
		final Date date02 = ((LogItem) o)._Date;
		return (_Date.compareTo(date02));
	}

	public String[] getText() {
		String[] sArray = null;
		if (_ObjectList.size() > 0) {
			final ArrayList<Object> list = new ArrayList<>();
			for (int i = 0; i < _ObjectList.size(); i++) {
				final Object o = _ObjectList.get(i);
				if (o instanceof String) {
					list.add(o);
				} else {
					if (o instanceof ExceptionItem) {
						list.addAll(((ExceptionItem) o).getTextAsList());
					}
				}
			}
			sArray = list.toArray(new String[list.size()]);
		}
		return sArray;
	}

}

class ExceptionItem {

	String _Message;
	Throwable _Throwable;

	public ExceptionItem(final String message, final Throwable throwable) {
		_Message = message;
		_Throwable = throwable;
	}

	public List<String> getTextAsList() {
		final List<String> list = new ArrayList<>();
		list.add(_Message);
		list.add(_Throwable.getMessage());
		list.add("     " + _Throwable);
		final StackTraceElement[] stackTraceElements = _Throwable.getStackTrace();
		for (final StackTraceElement stackTraceElement : stackTraceElements) {
			list.add("     " + stackTraceElement);
		}
		final Throwable causingThrowable = _Throwable.getCause();
		if (causingThrowable != null) {
			list.add("     caused by:");
			list.add("         " + causingThrowable);
			final StackTraceElement[] stackTraceElements2 = causingThrowable.getStackTrace();
			for (final StackTraceElement element : stackTraceElements2) {
				list.add("         " + element);
			}
		}
		return list;
	}

}
